@@ -406,8 +406,10 @@ ExtFunc int    stream_set_options(int, int, int);
 
 ExtFunc int	stream_set_buffer(int, int);
 ExtFunc int	stream_get_buffer_sz (int);
+ExtFunc int	stream_get_err(int);
 
 #ifdef HAVE_SSL
+ExtFunc	       SSL*    stream_get_ssl(int);
 ExtFunc        X509*   stream_get_server_certificate(int);
 ExtFunc	       char*   stream_get_ascii_server_certificate(int);
 #endif
@@ -437,6 +439,7 @@ ExtFunc int stream_isset(int, fd_set*);
 ExtFunc int bpf_server();
 ExtFunc int bpf_open_live(char*, char*);
 ExtFunc u_char* bpf_next(int, int *);
+ExtFunc u_char* bpf_next_tv(int, int *, struct timeval *);
 ExtFunc void bpf_close(int);
 ExtFunc int  bpf_datalink(int);
 
@@ -519,9 +522,6 @@ struct http_msg * http_share_send_recv_msg(struct arglist *, struct http_msg *);
 pid_t http_share_init(struct arglist *);
 int http_share_close(struct arglist *, pid_t);
 
-ExtFunc int http_recv_headers(int , char ** , int * );
-
-
 
 int os_send(int, void*, int, int);
 int os_recv(int, void*, int, int);
@@ -560,6 +560,7 @@ int internal_recv(int, char **, int *, int *);
 int internal_finished(int);
 	
 int fd_is_stream(int);
+int stream_pending(int);
 
 int send_fd(int, int);
 int recv_fd(int);
