@@ -21,61 +21,215 @@
 #define EXPORTING
 #include <includes.h>
 
-
-void * arg_get(arglst, name)
- struct arglist * arglst;
- char * name;
+/* 
+ * We use a hash of the argument name to speed up the lookups
+ * when calling arg_get_value()
+ */
+#define HASH_MAX 2713
+static int mkhash(const char * name)
 {
- char * toc;
- char **v;
- char * t;
- int id = 0;
- if(!arglst->toc)return(NULL);
- v = arglst->toc;
- toc = *v;
- t = strstr(toc, name);
- if(t)
+ int h = 0;
+ int i;
+ 
+ for(i=0;name[i] != '\0';i++)
  {
-  char *v = strchr(t, ':');
-  int i;
-  if(v)id = atoi(v+1);
-  for(i=1;i<id;i++)arglst = arglst->next;
-  return(arglst->value);
+  h = ((h * 128) + name[i]) % HASH_MAX;
  }
- return(NULL);
+ return h;
 }
 
-ExtFunc
-void arg_add_value(arglst, name, type, length, value)
+/*
+ * name_cache :
+ * 
+ * A lot of entries in our arglists have the same name.
+ * We use a caching system to avoid to allocate twice the same name
+ * 
+ * This saves about 300Kb of memory, with minimal performance impact
+ */
+struct name_cache {
+	char * name;
+	int occurences;
+	struct name_cache * next;
+	struct name_cache * prev;
+	};
+
+
+static int cache_inited = 0;
+static struct name_cache cache[HASH_MAX+1];
+
+
+static void cache_init()
+{
+ int i;
+ for(i=0;i<HASH_MAX+1;i++)
+ 	{
+	bzero(&(cache[i]), sizeof(cache[i]));
+	}
+ cache_inited = 1;
+}
+
+static struct name_cache * 
+cache_get_name(name)
+ char * name;
+{
+ struct name_cache * nc;
+ int h;
+ 
+ if(cache_inited == 0)
+ 	cache_init();
+	
+ if(!name)
+  return NULL;
+  
+ h = mkhash(name);
+
+ nc = cache[h].next;
+  
+ while(nc != NULL)
+ {
+  if(nc->name != NULL && 
+    !strcmp(nc->name, name))
+    	return nc;
+  else 
+  	nc = nc->next;
+ }
+ return NULL;
+}
+
+static struct name_cache *
+cache_add_name(name)
+ char * name;
+{
+ struct name_cache * nc;
+
+ int h;
+ 
+ if(name == NULL)
+  return NULL;
+ 
+ 
+ h = mkhash(name);
+ 
+ 
+ nc = emalloc(sizeof(struct name_cache));
+ nc->next = cache[h].next;
+ nc->prev = NULL;
+ nc->name = estrdup(name);
+ nc->occurences = 1;
+ if ( cache[h].next != NULL )
+  cache[h].next->prev = nc;
+ 
+ cache[h].next = nc;
+ 
+ return nc;
+}
+
+static char *
+cache_inc(name)
+ char * name;
+{
+ struct name_cache * nc = cache_get_name(name);
+ if(nc != NULL)
+  nc->occurences ++;
+ else
+   nc = cache_add_name(name);  
+ return nc->name;
+}
+
+static void 
+cache_dec(name)
+ char * name;
+{
+ struct name_cache* nc;
+
+ if(!name)
+  return;
+
+ nc  = cache_get_name(name);
+ if( nc == NULL)
+ {
+  /*
+  fprintf(stderr, "libnessus: cache_dec(): non-existant name\n");
+  */
+  return;
+ }
+ 
+ nc->occurences --;
+ if( nc->occurences == 0 ){
+ 	 int h = mkhash(name);
+ 	 efree(&nc->name);
+	 if(nc->next != NULL)
+	  nc->next->prev = nc->prev;
+	  
+	 if(nc->prev != NULL)
+	  nc->prev->next = nc->next;
+	 else
+	  cache[h].next = nc->next;
+	 
+	 efree(&nc);
+	}
+}
+
+
+
+
+ExtFunc void 
+arg_free_name(name)
+ char * name;
+{
+ cache_dec(name);
+}
+
+
+
+
+ 
+
+ExtFunc void 
+arg_add_value(arglst, name, type, length, value)
   struct arglist * arglst;
   const char * name;
   int type;
   long length;
   void * value;
 {
-	int id;
 	if(!arglst)return;
-
-	/* Go to the end of our list... */
-	while(arglst->next && arglst->next->next)arglst = arglst->next;
-	id = arglst->id+1;
-	if(arglst->next)arglst = arglst->next;
+	while(arglst->next)arglst = arglst->next;
 	
 	if (type == ARG_STRUCT) {
     	 void* new_val = emalloc(length);
-     	memcpy(new_val, value, length);
-    	value = new_val;
+     	 memcpy(new_val, value, length);
+    	 value = new_val;
    	}
 
-	arglst->name = estrdup(name);
+	arglst->name = cache_inc(name);
 	arglst->value = value;
 	arglst->length = length;
 	arglst->type = type;
 	arglst->next = emalloc(sizeof(struct arglist));
-	arglst->next->next = NULL;
+	arglst->hash = mkhash(arglst->name);
 }
 
 
+static struct arglist * arg_get(struct arglist * arg, const char * name)
+{
+ int h = mkhash(name);
+ if(arg == NULL)
+  return NULL;
+ 
+ while(arg->next != NULL)
+ {
+  if(arg->hash == h && strcmp(arg->name, name) == 0)
+    return arg;
+  else
+   arg = arg->next;
+ }
+ return NULL;
+}
+
+
+
+
 ExtFunc int 
 arg_set_value(arglst, name, length, value)
  struct arglist * arglst;
@@ -83,11 +237,13 @@ arg_set_value(arglst, name, length, value)
  long length;
  void *value;
 {
-  /* We look for the asked name */
-  while(arglst->next && strcmp(arglst->name, name))
-    arglst = arglst->next;
+ 
+ if(name == NULL)
+  return -1;
   
-  if(arglst->next)
+ arglst = arg_get(arglst, name);
+  
+  if(arglst != NULL)
     {
       if (arglst->type == ARG_STRUCT) {
 	void* new_val = emalloc(length);
@@ -97,9 +253,9 @@ arg_set_value(arglst, name, length, value)
       }
       arglst->value = value;
       arglst->length = length;
-      return(0);
+      return 0;
     }
-  else return(-1); 
+  else return -1; 
 }
 
 ExtFunc int 
@@ -108,8 +264,10 @@ arg_set_type(arglst, name, type)
  const char * name;
  int type;
 {
-  while(arglst->next && strcmp(name, arglst->name))arglst=arglst->next;
-  if(!(arglst->next))return(-1);
+  arglst = arg_get(arglst, name);
+  if(arglst == NULL)
+   return -1;
+   
   if (arglst->type == ARG_STRUCT  &&  type != ARG_STRUCT) {
     efree(&arglst->value);
   }
@@ -123,15 +281,15 @@ arg_get_value(args, name)
  struct arglist * args;
  const char * name;
 {
- int flag=0;
- if(!args)return(NULL); 
-  while(args && args->next && !flag)
-  	{
-        if(args->name && !strcmp(name, args->name))flag=1;
-  	else args=args->next;
-  	}
-  if(flag && args && args->next)return(args->value);
-  else return(NULL);
+
+  if(args == NULL)
+   return NULL;
+  
+  args = arg_get(args, name);
+  if(args == NULL)
+   return NULL;
+  else  
+  return(args->value);
 }
 
 
@@ -140,10 +298,11 @@ arg_get_length(args,name)
  struct arglist * args;
  const char * name;
 {
-  while(args->next && strcmp(name, args->name))args=args->next;
-  if(args->next)
+  args = arg_get(args, name);
+  if(args != NULL)
     return(args->length);
-  else return(0);
+  else 
+    return 0;
 }
 
 
@@ -152,10 +311,11 @@ arg_get_type(args,name)
  struct arglist * args;
  const char * name;
 {
-  while(args->next && strcmp(name, args->name))args=args->next;
-  if(args->next)
+ args = arg_get(args, name);
+ if( args != NULL )
     return(args->type);
-  else return(-1);
+  else 
+    return -1;
 }
 
 
@@ -163,11 +323,15 @@ ExtFunc void arg_dup(dst, src)
  struct arglist * dst;
  struct arglist * src;
 {
- while(src && src->next)
+ if(!src)
+  return;
+  
+ while(src->next)
  {
-  dst->name = estrdup(src->name);
+  dst->name = cache_inc(src->name);
   dst->type = src->type;
   dst->length = src->length;
+  dst->hash = src->hash;
   switch(src->type)
   {
    case ARG_INT :
@@ -214,8 +378,9 @@ arg_dump(args, level)
 		return;
 	}
 	
-	while(args && args->next)
-	{
+	if(args)
+	 while(args->next)
+	 {
 		switch(args->type)
 		{
 			case ARG_STRING :
@@ -253,7 +418,7 @@ arg_free(arg)
  while(arg)
  {
   struct arglist * next = arg->next;
-  if (arg->name) efree (&(arg->name)) ;
+  cache_dec(arg->name);
   efree(&arg);
   arg = next;
  }
@@ -278,7 +443,7 @@ ExtFunc void arg_free_all(arg)
     efree(&arg->value);
     break;
   }
-  if (arg->name) efree (&(arg->name)) ;
+  cache_dec(arg->name);
   efree(&arg);
   arg = next;
  }
