@@ -21,19 +21,17 @@
    Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Modified for use with Nessus by <jordan@mjh.teddy-net.com>
-   $Id: getopt.c,v 1.6.2.1 2001/08/23 13:54:53 renaud Exp $ */
+   $Id: getopt.c,v 1.10 2001/10/16 18:48:55 renaud Exp $ */
 
 /* NOTE!!!  AIX requires this to be the first thing in the file.
    Do not put ANYTHING before it!  */
+      
 #if !defined (__GNUC__) && defined (_AIX)
 #pragma alloca
 #endif
 
-#include <config.h>
+#include <includes.h>
 
-#ifdef _WIN32
-#include "config.w32"
-#endif
 
 /***********************/
 #ifndef HAVE_GETOPT_LONG
@@ -95,7 +93,7 @@ char *alloca ();
 #define __alloca _alloca
 #endif
 #ifndef __alloca
-#error You loose -- alloca() is needed to compile this source
+#error alloca() is needed to compile this source
 #endif
 
 #if !defined (__STDC__) && !defined (const)
@@ -133,8 +131,11 @@ char *alloca ();
    the argument value is returned here.
    Also, when `ordering' is RETURN_IN_ORDER,
    each non-option ARGV-element is returned here.  */
-
+#ifdef HAVE_OPTIND
+extern char* optarg;
+#else
 char *optarg = 0;
+#endif
 
 /* Index in ARGV of the next element to be scanned.
    This is used for communication to and from the caller
@@ -149,7 +150,11 @@ char *optarg = 0;
    how much of ARGV has been scanned so far.  */
 
 /* XXX 1003.2 says this must be 1 before any call.  */
+#ifdef HAVE_OPTIND
+extern int optind;
+#else
 int optind = 0;
+#endif
 
 /* The next char to be scanned in the option-element
    in which the last option character we returned was found.
@@ -163,13 +168,22 @@ static char *nextchar;
 /* Callers store zero here to inhibit the error message
    for unrecognized options.  */
 
+#ifdef HAVE_OPTIND
+extern int opterr;
+#else
 int opterr = 1;
+#endif
 
 /* Set to an option character which was unrecognized.
    This must be initialized on some systems to avoid linking in the
    system's own getopt implementation.  */
 
+#ifdef HAVE_OPTIND
+extern int optint;
+#else
 int optopt = '?';
+#endif
+
 
 /* Describe how to deal with options that follow non-option ARGV-elements.
 
