@@ -38,7 +38,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /usr/local/cvs/nessus-libraries/libpcap-nessus/pcap-dlpi.c,v 1.6 2003/02/06 20:28:08 renaud Exp $ (LBL)";
+    "@(#) $Header: /usr/local/cvs/nessus-libraries/libpcap-nessus/pcap-dlpi.c,v 1.6.4.1 2007/06/05 13:28:01 renaud Exp $ (LBL)";
 #endif
 
 #include <sys/types.h>
@@ -248,11 +248,15 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	/*
 	** Determine device and ppa
 	*/
-	cp = strpbrk(device, "0123456789");
-	if (cp == NULL) {
+	cp = device + strlen(device) - 1;
+        if ( *cp < '0' || *cp > '9' ) {
 		sprintf(ebuf, "%s missing unit number", device);
 		goto bad;
 	}
+
+        while ( cp - 1 >= device && *(cp - 1) >= '0' && *(cp - 1) <= '9' )
+		cp--;
+
 	ppa = strtol(cp, &eos, 10);
 	if (*eos != '\0') {
 		sprintf(ebuf, "%s bad unit number", device);
